/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
package org.greenstone.gatherer.cdm;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import org.greenstone.gatherer.DebugStream;
import org.greenstone.gatherer.Dictionary;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.gui.GLIButton;
import org.greenstone.gatherer.util.CheckList;
import org.greenstone.gatherer.util.Utility;
import org.w3c.dom.*;


/** This class maintains a list of indexes partitions for the purpose of defining subcollections.
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.4
 */
public class SubcollectionIndexManager
    extends DOMProxyListModel
{
    static final private Dimension FIELD_SIZE = new Dimension(200, 30);

    private Control controls;
    private DOMProxyListModel model;
    private SubcollectionIndex default_index;


    /** Constructor. */
    public SubcollectionIndexManager(Element subindexes)
    {
	super(subindexes, CollectionConfiguration.INDEX_ELEMENT, new SubcollectionIndex());
	DebugStream.println("SubcollectionIndexManager: " + getSize() + " subcollection indexes parsed.");
	model = this;

	// Parse and retrieve the default index
	NodeList default_index_elements = CollectionDesignManager.collect_config.getDocumentElement().getElementsByTagName(CollectionConfiguration.SUBCOLLECTION_DEFAULT_INDEX_ELEMENT);
	if (default_index_elements.getLength() > 0) {
	    default_index = new SubcollectionIndex((Element) default_index_elements.item(0));
	}
    }


    /** Method to add a subindex.
     * @param subindex a SubcollectionIndex
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     */
    private void addSubcollectionIndex(SubcollectionIndex subcollection_index)
    {
	if (!contains(subcollection_index)) {
	    add(getSize(), subcollection_index);
	    Gatherer.c_man.configurationChanged();
	}
    }


    public void destroy()
    {
	if (controls != null) {
	    controls.destroy();
	    controls = null;
	}
	default_index = null;
	model = null;
    }


    public Control getControls()
    {
	if (controls == null) {
	    controls = new SubcollectionIndexControls();
	}
	return controls;
    }


    /** Method to get all of the subindexes set.
     * @return an ArrayList containing all the defined indexes
     */
    public ArrayList getSubcollectionIndexes()
    {
	return children();
    }


    private int moveSubcollectionIndex(SubcollectionIndex subcollection_index, boolean move_up)
    {
	// Determine the current position of the subcollection index
	int position = indexOf(subcollection_index);
	int new_position;

	// Attempt to move the subcollection index up
	if (move_up) {
	    // Check it's not already at the top
	    if (position == 0) {
		return position;
	    }

	    // This automatically removes the index first, as an Element can only exist once in a particular document
	    new_position = position - 1;
	    addBefore(subcollection_index, (SubcollectionIndex) getElementAt(new_position));
	}

	// Attempt to move the subcollection index down
	else {
	    // Check it's not already at the bottom
	    if (position == (getSize()) - 1) {
		return position;
	    }

	    // This automatically removes the index first, as an Element can only exist once in a particular document
	    new_position = position + 1;
	    addAfter(subcollection_index, (SubcollectionIndex) getElementAt(new_position));
	}

	// Schedule the collection for saving
	Gatherer.c_man.configurationChanged();
	return new_position;
    }


    /** Method to remove a certain subcollection index. */
    private void removeSubcollectionIndex(SubcollectionIndex subcollection_index)
    {
	if (subcollection_index != null) {
	    // Remove any current metadata from this index
	    CollectionDesignManager.collectionmeta_manager.removeMetadata("." + subcollection_index.getID());

	    // Check if the index removed happens to be the default index
	    if (default_index != null && default_index.equals(subcollection_index)) {
		// If so our first solution is to set the first subcollection index to be the default
		if (getSize() > 0) {
		    setDefaultSubcollectionIndex((SubcollectionIndex) getElementAt(0));
		}
		else {
		    default_index.setAssigned(false);
		}
	    }

	    // Remove the index
	    remove(subcollection_index);
	    Gatherer.c_man.configurationChanged();
	}
    }


    /** Method to remove all of the subindexes that contain a certain subcollection.
     * @param subcollection the Subcollection that has been removed
     * @see org.greenstone.gatherer.cdm.Subcollection
     * @see org.greenstone.gatherer.cdm.SubcollectionIndex
     */
    public void removeSubcollectionIndexes(Subcollection subcollection)
    {
	String subcollection_name = subcollection.getName();
	int size = getSize();
	for(int i = size - 1; i >= 0; i--) {
	    SubcollectionIndex subcollection_index = (SubcollectionIndex) getElementAt(i);
	    if (subcollection_index.getSources().contains(subcollection_name)) {
		removeSubcollectionIndex(subcollection_index);
	    }
	}
    }


    /** Method to set the default subcollection index.
     * @param index The <strong>SubcollectionIndex</strong> to use as the default index.
     * @see org.greenstone.gatherer.Gatherer
     * @see org.greenstone.gatherer.collection.CollectionManager
     * @see org.greenstone.gatherer.cdm.SubcollectionIndex
     */
    private void setDefaultSubcollectionIndex(SubcollectionIndex subcollection_index)
    {
 	if (subcollection_index != null) {
 	    if (default_index == null) {
		// Create the default index element, and place immediately after indexes element.
		Element default_index_element = root.getOwnerDocument().createElement(CollectionConfiguration.SUBCOLLECTION_DEFAULT_INDEX_ELEMENT);
		default_index = new SubcollectionIndex(default_index_element);
		Node target_node = CollectionConfiguration.findInsertionPoint(default_index_element);
		if (target_node != null) {
		    root.getOwnerDocument().getDocumentElement().insertBefore(default_index_element, target_node);
		}
		else {
		    root.getOwnerDocument().getDocumentElement().appendChild(default_index_element);
		}
 	    }
 	    default_index.setAssigned(true);
 	    default_index.setSources(subcollection_index.getSources());
 	}
 	else {
 	    if (default_index != null) {
 		default_index.setAssigned(false);
 	    }
 	}
 	Gatherer.c_man.configurationChanged();
    }


    /** This class creates a set of controls for editing the indexes. */
    private class SubcollectionIndexControls
	extends JPanel
	implements Control
    {
	private CheckList source_list;
	private JButton add_button;
	private JButton move_down_button;
	private JButton move_up_button;
	private JButton remove_button;
	private JButton replace_button;
	private JButton set_default_button;
	private JList subcollection_index_list;
	private JTextField subcollection_index_name_textfield;


	/** Constructor.
	 */
	public SubcollectionIndexControls()
	{
	    super();

	    ArrayList sources = new ArrayList();
	    ListModel source_model = CollectionDesignManager.subcollection_manager;
	    for (int i = 0; i < source_model.getSize(); i++) {
		sources.add(source_model.getElementAt(i));
	    }

	    // Creation
	    JPanel assigned_indexes_pane = new JPanel();
	    JLabel index_label = new JLabel();
	    Dictionary.registerText(index_label, "CDM.SubcollectionIndexManager.Subindexes");
 	    subcollection_index_list = new JList(model);
	    subcollection_index_list.setCellRenderer(new SubcollectionIndexListCellRenderer());
 	    subcollection_index_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
	    subcollection_index_list.setVisibleRowCount(2);

	    JPanel movement_pane = new JPanel();
	    move_up_button = new JButton("", Utility.getImage("arrow-up.gif"));
	    move_up_button.setEnabled(true);
	    move_up_button.setMnemonic(KeyEvent.VK_U);
	    Dictionary.registerBoth(move_up_button, "CDM.Move.Move_Up", "CDM.Move.Move_Up_Tooltip");

	    move_down_button = new JButton("", Utility.getImage("arrow-down.gif"));
	    move_down_button.setEnabled(true);
	    move_down_button.setMnemonic(KeyEvent.VK_D);
	    Dictionary.registerBoth(move_down_button, "CDM.Move.Move_Down", "CDM.Move.Move_Down_Tooltip");

	    set_default_button = new GLIButton();
	    set_default_button.setEnabled(false);
	    set_default_button.setMnemonic(KeyEvent.VK_S);
	    Dictionary.registerBoth(set_default_button, "CDM.SubcollectionIndexManager.Set_Default_Subindex", "CDM.SubcollectionIndexManager.Set_Default_Subindex_Tooltip");

	    JPanel index_pane = new JPanel();
	    JPanel details_pane = new JPanel();
	    JPanel labels_pane = new JPanel();
	    JPanel boxes_pane = new JPanel();
	    JPanel content_pane = new JPanel();

	    JLabel name_label = new JLabel();
	    Dictionary.registerText(name_label, "CDM.SubcollectionIndexManager.PartitionName");
	    subcollection_index_name_textfield = new JTextField();
	    subcollection_index_name_textfield.setPreferredSize(FIELD_SIZE);
	    Dictionary.registerTooltip(subcollection_index_name_textfield, "CDM.SubcollectionIndexManager.PartitionName_Tooltip");

	    JLabel source_label = new JLabel();
	    Dictionary.registerText(source_label, "CDM.SubcollectionIndexManager.Source");
	    source_list = new CheckList(false);
	    source_list.setListData(sources);
	    Dictionary.registerTooltip(source_list, "CDM.SubcollectionIndexManager.Source_Tooltip");

	    JPanel button_pane = new JPanel();
	    add_button = new GLIButton();
	    add_button.setEnabled(false);
	    add_button.setMnemonic(KeyEvent.VK_A);
	    Dictionary.registerBoth(add_button, "CDM.SubcollectionIndexManager.Add_Subindex", "CDM.SubcollectionIndexManager.Add_Subindex_Tooltip");

	    remove_button = new GLIButton();
	    remove_button.setEnabled(false);
	    remove_button.setMnemonic(KeyEvent.VK_R);
	    Dictionary.registerBoth(remove_button, "CDM.SubcollectionIndexManager.Remove_Subindex", "CDM.SubcollectionIndexManager.Remove_Subindex_Tooltip");

	    replace_button = new GLIButton();
	    replace_button.setEnabled(false);
	    replace_button.setMnemonic(KeyEvent.VK_P);
	    Dictionary.registerBoth(replace_button, "CDM.SubcollectionIndexManager.Replace_Subindex", "CDM.SubcollectionIndexManager.Replace_Subindex_Tooltip");

	    // Listeners
	    add_button.addActionListener(new AddListener());
	    move_down_button.addActionListener(new MoveListener(false));
	    move_up_button.addActionListener(new MoveListener(true));
	    remove_button.addActionListener(new RemoveListener());
	    replace_button.addActionListener(new ReplaceListener());
	    set_default_button.addActionListener(new SetDefaultListener());
	    subcollection_index_name_textfield.getDocument().addDocumentListener(new NameListener());
	    subcollection_index_list.addListSelectionListener(new SubcollectionIndexListListener());
	    source_list.addListSelectionListener(new SourceListListener());

	    // Layout
	    movement_pane.setBorder(BorderFactory.createEmptyBorder(0,2,0,0));
	    movement_pane.setLayout(new GridLayout(3,1));
	    movement_pane.add(move_up_button);
	    movement_pane.add(move_down_button);
	    movement_pane.add(set_default_button);

	    assigned_indexes_pane.setBorder(BorderFactory.createEmptyBorder(0,0,5,0));
	    assigned_indexes_pane.setLayout(new BorderLayout());
	    assigned_indexes_pane.add(index_label, BorderLayout.NORTH);
	    assigned_indexes_pane.add(new JScrollPane(subcollection_index_list), BorderLayout.CENTER);
	    assigned_indexes_pane.add(movement_pane, BorderLayout.EAST);

	    labels_pane.setLayout(new BorderLayout());
	    labels_pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 10, 5));
	    labels_pane.add(name_label, BorderLayout.NORTH);
	    labels_pane.add(source_label, BorderLayout.CENTER);
	    
	    boxes_pane.setLayout(new BorderLayout());
	    boxes_pane.add(subcollection_index_name_textfield, BorderLayout.NORTH);
	    boxes_pane.add(new JScrollPane(source_list), BorderLayout.CENTER);
	    
	    details_pane.setLayout(new BorderLayout());
	    details_pane.add(labels_pane, BorderLayout.WEST);
	    details_pane.add(boxes_pane, BorderLayout.CENTER);

	    button_pane.setLayout(new GridLayout(1,3));
	    button_pane.add(add_button);
	    button_pane.add(replace_button);
	    button_pane.add(remove_button);

	    index_pane.setLayout(new BorderLayout());
	    index_pane.add(details_pane, BorderLayout.CENTER);
	    index_pane.add(button_pane, BorderLayout.SOUTH);

	    content_pane.setLayout(new BorderLayout());
	    content_pane.add(assigned_indexes_pane, BorderLayout.NORTH);
	    content_pane.add(index_pane, BorderLayout.CENTER);

	    setBorder(BorderFactory.createEmptyBorder(5,5,5,5));
	    setLayout(new BorderLayout());
	    add(content_pane, BorderLayout.CENTER);
	}


	public void destroy()
	{
	}


	public void gainFocus()
	{
 	    // Reload the source list
 	    ArrayList sources = new ArrayList();
 	    ListModel source_model = CollectionDesignManager.subcollection_manager;
 	    for (int i = 0; i < source_model.getSize(); i++) {
 		sources.add(source_model.getElementAt(i));
 	    }
 	    source_list.setListData(sources);

 	    // Refresh the subcollection index list
 	    subcollection_index_list.updateUI();

 	    // If there is one selected, fill in the controls
 	    updateControlsWithSelectedIndex();
	}


	public void loseFocus()
	{
	}


	private void updateControlsWithSelectedIndex()
	{
	    SubcollectionIndex selected_index = (SubcollectionIndex) subcollection_index_list.getSelectedValue();
	    if (selected_index == null) {
		return;
	    }

	    // Display the selected subcollection index's name
	    String id = selected_index.getID();
	    String name = CollectionDesignManager.collectionmeta_manager.getMetadatum("." + id).getValue(true);
	    subcollection_index_name_textfield.setText(name);

	    // Display the selected subcollection index's sources
	    source_list.clearTicked();
	    source_list.setTickedObjects(selected_index.getSources().toArray());
	}


	private void validateControls()
	{
	    boolean add_enabled = false;
	    boolean replace_enabled = false;

	    // Indexes must have a name
	    if (subcollection_index_name_textfield.getText().length() == 0) {
		add_enabled = false;
	    }
	    // Can't add a new index if no sources are selected
	    else if (source_list.isNothingTicked()) {
		add_enabled = false;
	    }
	    // If we get this far, create a dummy index and see if it's already assigned in the collection
	    else {
 		ArrayList sources = source_list.getTicked();
 		SubcollectionIndex subcollection_index = new SubcollectionIndex(sources.toArray());

		// Subcollection index already exists: can't add, but can replace if the name has changed
 		if (model.contains(subcollection_index)) {
 		    add_enabled = false;

 		    // Here we need to check if we have changed the name - if so, we can enable the replace button
		    if (subcollection_index_list.getSelectedIndex() != -1) {
			String name = subcollection_index_name_textfield.getText();
 			SubcollectionIndex selected_index = (SubcollectionIndex) subcollection_index_list.getSelectedValue();
 			String selected_name = CollectionDesignManager.collectionmeta_manager.getMetadatum("." + selected_index.getID()).getValue(true);
 			if (!name.equals(selected_name)) {
 			    replace_enabled = true;
 			}
 		    }
		}

		// Subcollection index doesn't already exist: can add, or replace if there is something selected
		else {
 		    add_enabled = true;
 		    if (subcollection_index_list.getSelectedIndex() != -1) {
 			replace_enabled = true;
 		    }
 		}
	    }

	    // We should now know the add_button state
	    add_button.setEnabled(add_enabled);
	    replace_button.setEnabled(replace_enabled);
	}


	private class AddListener
	    implements ActionListener
	{
	    public void actionPerformed(ActionEvent event)
	    {
		String name = subcollection_index_name_textfield.getText();
		if (!source_list.isNothingTicked() && name.length() != 0) {
		    ArrayList sources = source_list.getTicked();
		    SubcollectionIndex subcollection_index = new SubcollectionIndex(sources.toArray());

		    // Before we add the index to the model, we have to add the collection metadata for this
		    CollectionMeta metadatum = new CollectionMeta("." + subcollection_index.getID());
		    metadatum.setValue(name);
 		    CollectionDesignManager.collectionmeta_manager.addMetadatum(metadatum);

		    // Finally, add the subcollection index
		    addSubcollectionIndex(subcollection_index);
		    subcollection_index_list.setSelectedIndex(model.getSize() - 1);
		}
	    }
	}


	private class MoveListener
	    implements ActionListener
	{
	    private boolean move_up;

	    public MoveListener(boolean move_up)
	    {
		this.move_up = move_up;
	    }

	    public void actionPerformed(ActionEvent event)
	    {
		// Retrieve the selected subcollection index
		SubcollectionIndex subcollection_index = (SubcollectionIndex) subcollection_index_list.getSelectedValue();
		if (subcollection_index != null) {
		    int new_position = moveSubcollectionIndex(subcollection_index, move_up);
		    // Ensure the subcollection index that moved is still selected
		    subcollection_index_list.setSelectedIndex(new_position);
		}
	    }
	}


	/** Listens for key presses within the name field, and enabled or disables controls as appropriate. */
	private class NameListener
	    implements DocumentListener
	{
	    public void changedUpdate(DocumentEvent e)
	    {
		validateControls();
	    }

	    public void insertUpdate(DocumentEvent e)
	    {
		validateControls();
	    }

	    public void removeUpdate(DocumentEvent e)
	    {
		validateControls();
	    }
	}


	private class RemoveListener
	    implements ActionListener
	{
	    public void actionPerformed(ActionEvent event)
	    {
		int i = subcollection_index_list.getSelectedIndex();
		if (i != -1) {
		    removeSubcollectionIndex((SubcollectionIndex) subcollection_index_list.getSelectedValue());
		}

		int size = subcollection_index_list.getModel().getSize();
		if (i == size) {
		    i--;
		}
		subcollection_index_list.setSelectedIndex(i);
		validateControls();
	    }
	}


	private class ReplaceListener
	    implements ActionListener
	{
	    public void actionPerformed(ActionEvent event)
	    {
		if (subcollection_index_list.isSelectionEmpty()) {
		    // This should never happen, but just in case...
		    replace_button.setEnabled(false);
		    return;
		}

		// We'll just remove the old one and add the new one
		removeSubcollectionIndex((SubcollectionIndex) subcollection_index_list.getSelectedValue());
		replace_button.setEnabled(false);
		add_button.setEnabled(true);
		add_button.doClick();
		add_button.setEnabled(false);
	    }
	}


	private class SetDefaultListener
	    implements ActionListener
	{
	    public void actionPerformed(ActionEvent event)
	    {
		SubcollectionIndex subcollection_index = (SubcollectionIndex) subcollection_index_list.getSelectedValue();
		if (subcollection_index != null) {
		    setDefaultSubcollectionIndex(subcollection_index);
		    // This should cause a repaint of just the desired row
		    subcollection_index_list.setSelectedValue(subcollection_index, true);
		}
		set_default_button.setEnabled(false);
	    }
	}


	private class SourceListListener
	    implements ListSelectionListener
	{
	    public void valueChanged(ListSelectionEvent event)
	    {
		validateControls();
	    }
	}


	/** Listens for selections within the list on the SubcollectionIndexManager controls, and if a change is detected enables, or disables, controls appropriately. */
	private class SubcollectionIndexListListener
	    implements ListSelectionListener
	{
	    /** This method is called whenever the source list selection changes. When it does we need to fill in the various parts of the list description panel
	     * @param event A <strong>ListSelectionEvent</strong> containing further information about the list selection.
	     */
	    public void valueChanged(ListSelectionEvent event)
	    {
		if (event.getValueIsAdjusting()) {
		    return;
		}

 		int i = subcollection_index_list.getSelectedIndex();
 		int size = subcollection_index_list.getModel().getSize();
 		if (i < 0 || i >= size) {
 		    move_down_button.setEnabled(false);
 		    move_up_button.setEnabled(false);
 		    remove_button.setEnabled(false);
 		    replace_button.setEnabled(false);
 		    set_default_button.setEnabled(false);
 		    return;
 		}

 		// Enable the buttons appropriately
 		remove_button.setEnabled(true);
 		SubcollectionIndex selected_index = (SubcollectionIndex) subcollection_index_list.getSelectedValue();
 		set_default_button.setEnabled(default_index == null || !default_index.equals(selected_index));
 		if (i > 0) {
 		    move_up_button.setEnabled(true);
 		}
 		else {
 		    move_up_button.setEnabled(false);
 		}
 		if (i < size-1){
 		    move_down_button.setEnabled(true);
 		}
 		else {
 		    move_down_button.setEnabled(false);
 		}

 		// Need to fill in the rest of the bits
		updateControlsWithSelectedIndex();
	    }
	}


	private class SubcollectionIndexListCellRenderer
	    extends DefaultListCellRenderer
	{
	    /** Return a component that has been configured to display the specified value. */
	    public Component getListCellRendererComponent(JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
		JLabel component = (JLabel) super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
		if (default_index != null && default_index.equals(value)) {
		    component.setText(component.getText() + " " + Dictionary.get("CDM.SubcollectionIndexManager.Default_Partition_Indicator"));
		}
		return component;
	    }
	}
    }
}
